package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	db2 "applet/app/db/gim"
	"applet/app/db/gim/model"
	model2 "applet/app/db/model"
	md2 "applet/app/md"
	utils2 "applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

const UserRegisterConsumeForMyRecommenderKey = "UserRegisterConsumeForMyRecommender" // 用户访问记录 uid index

func UserRegisterConsumeForMyRecommender(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>UserRegisterConsumeForMyRecommender>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleUserRegisterConsumeForMyRecommender(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)

				utils2.FilePutContents("UserRegisterConsumeForMyRecommender_err", "[err]:"+err.Error())
				if err.Error() == "用户暂未注册im系统" {
					//TODO::重新推回队列末尾，避免造成队列堵塞
					var msg *md2.UserRegisterMessageStructForMyRecommender
					json.Unmarshal(res.Body, &msg)
					ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
				}
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleUserRegisterConsumeForMyRecommender(msgData []byte) error {
	//time.Sleep(time.Duration(20) * time.Second)
	//1、解析mq中queue的数据结构体
	var msg *md2.UserRegisterMessageStructForMyRecommender
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}

	//TODO::判断是否已操作过
	get, err1 := cache.GetBit(UserRegisterConsumeForMyRecommenderKey, msg.Uid)
	if err1 != nil || get == 0 {
		_, err1 = cache.SetBit(UserRegisterConsumeForMyRecommenderKey, msg.Uid, 1)
		if err1 != nil {
			return err1
		}
	}
	if get == 1 {
		return nil
	}

	//2、查找用户对应im系统中的数据
	gimUser, err := db2.UserGetOneByParams(db.DbIm, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}

	if gimUser == nil {
		fmt.Println("===========================", msg.Phone)
		user, err := db.UserFindByMobile(db.Db, strconv.FormatInt(msg.Phone, 10))
		if err != nil {
			return err
		}

		gimUserId, err := db2.UserInsert(db.DbIm, &model.User{
			PhoneNumber:        strconv.FormatInt(msg.Phone, 10),
			Nickname:           user.Nickname,
			AvatarUrl:          user.Avatar,
			Sex:                user.Sex,
			CreateTime:         time.Now().Format("2006-01-02 15:04:05"),
			UpdateTime:         time.Now().Format("2006-01-02 15:04:05"),
			IsAutoAddedFriends: 0,
		})
		if err != nil {
			return errors.New("用户暂未注册im系统")
		}
		gimUser, err = db2.UserGetOneByParams(db.DbIm, map[string]interface{}{
			"key":   "id",
			"value": gimUserId,
		})
		if gimUser == nil {
			return errors.New("用户暂未注册im系统")
		}
	}

	//2、查找是否有群
	var fansGroup model2.ImGroup
	fansGroups, err := db.ImGroupFindByParamsByKind(db.Db, 3, map[string]interface{}{
		"key":   "uid",
		"value": msg.RecommenderUid,
	})
	if err != nil {
		return err
	}
	for _, group := range *fansGroups {
		//统计当前群有多少人
		userGroups, err := db2.GroupUserFindByParams(db.DbIm, map[string]interface{}{
			"key":   "group_id",
			"value": group.GroupId,
		})
		if err != nil {
			return err
		}
		if len(*userGroups) < 1000 {
			fansGroup = group
		}
	}

	if fansGroup.GroupId == 0 {
		recommenderUser, err := db.UserFindByID(db.Db, msg.RecommenderUid)
		if err != nil {
			return err
		}
		recommenderGimUser, err := db2.UserGetOneByParams(db.DbIm, map[string]interface{}{
			"key":   "phone_number",
			"value": recommenderUser.Phone,
		})
		if recommenderGimUser == nil {
			return errors.New("当前推荐人信息有误，请联系管理员！！！")
		}

		groupName := "我的粉丝【" + utils2.AnyToString(len(*fansGroups)+1) + "】群"
		//3、创建群
		resp, err := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).CreateGroup(utils.GetCtx("", strconv.FormatInt(recommenderGimUser.Id, 10), "0"), &pb.CreateGroupReq{
			Name:      groupName,
			AvatarUrl: "",
			//Introduction: "官方群",
			Introduction: "",
			Extra:        "",
			MemberIds:    []int64{},
		})
		if err != nil {
			return err
		}

		//插入 im_group 记录
		now := time.Now()
		_, err = db.ImGroupInsert(db.Db, &model2.ImGroup{
			Kind:       3,
			Uid:        int(msg.RecommenderUid),
			GroupId:    int(resp.GroupId),
			IsFull:     0,
			Name:       groupName,
			CreateTime: now,
			UpdateTime: now,
		})
		if err != nil {
			return err
		}
		tempFansGroup, err := db.ImGroupGetOneByParams(db.Db, 2, map[string]interface{}{
			"key":   "group_id",
			"value": resp.GroupId,
		})
		if err != nil {
			return err
		}
		fansGroup = *tempFansGroup
	}

	userGroup, err2 := db2.GroupUserGetOneByParams(db.DbIm, 1, map[string]interface{}{
		"key":   "group_id",
		"value": fansGroup.GroupId,
	})
	if err2 != nil {
		return err2
	}
	if userGroup == nil {
		return errors.New("当前推荐人群暂未设置群主，请联系管理员！！！")
	}

	//加入群
	fmt.Println(fmt.Sprintf("=======GroupId======%d==============", fansGroup.GroupId))
	fmt.Println(fmt.Sprintf("=======gimUserId======%d==============", gimUser.Id))
	_, err = utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(
		utils.GetCtx("", strconv.FormatInt(userGroup.UserId, 10), ""),
		&pb.AddGroupMembersReq{
			GroupId: int64(fansGroup.GroupId),
			UserIds: []int64{gimUser.Id},
		})
	return nil
}
