package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	"applet/app/db/gim"
	imModel "applet/app/db/gim/model"
	"applet/app/db/model"
	md2 "applet/app/md"
	utils2 "applet/app/utils"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

const UserRegisterConsumeForOfficialKey = "UserRegisterConsumeForOfficial" // 用户访问记录 uid index

func UserRegisterConsumeForOfficial(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>UserRegisterConsumeForOfficial>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleUserRegisterConsumeForOfficial(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)

				utils2.FilePutContents("UserRegisterConsumeForOfficial_err", "[err]:"+err.Error())
				if err.Error() == "用户暂未注册im系统" {
					//TODO::重新推回队列末尾，避免造成队列堵塞
					var msg *md2.UserRegisterMessageStructForOfficial
					json.Unmarshal(res.Body, &msg)
					ch.Publish(queue.ExchangeName, msg, queue.RoutKey)
				}
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleUserRegisterConsumeForOfficial(msgData []byte) error {
	//time.Sleep(time.Duration(1) * time.Second)
	//1、解析mq中queue的数据结构体
	var msg *md2.UserRegisterMessageStructForOfficial
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	//TODO::判断是否已操作过
	get, err1 := cache.GetBit(UserRegisterConsumeForOfficialKey, msg.Uid)
	if err1 != nil || get == 0 {
		_, err1 = cache.SetBit(UserRegisterConsumeForOfficialKey, msg.Uid, 1)
		if err1 != nil {
			return err1
		}
	}
	if get == 1 {
		return nil
	}

	//2、查找用户对应im系统中的数据
	gimUser, err := gim.UserGetOneByParams(db.DbIm, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}
	if gimUser == nil {
		fmt.Println("===========================", msg.Phone)
		user, err := db.UserFindByMobile(db.Db, strconv.FormatInt(msg.Phone, 10))
		if err != nil {
			return err
		}
		if user == nil {
			return nil
		}
		gimUserId, err := gim.UserInsert(db.DbIm, &imModel.User{
			PhoneNumber:        strconv.FormatInt(msg.Phone, 10),
			Nickname:           user.Nickname,
			AvatarUrl:          user.Avatar,
			Sex:                user.Sex,
			CreateTime:         time.Now().Format("2006-01-02 15:04:05"),
			UpdateTime:         time.Now().Format("2006-01-02 15:04:05"),
			IsAutoAddedFriends: 0,
		})
		if err != nil {
			return errors.New("用户暂未注册im系统")
		}
		gimUser, err = gim.UserGetOneByParams(db.DbIm, map[string]interface{}{
			"key":   "id",
			"value": gimUserId,
		})
		if gimUser == nil {
			return errors.New("用户暂未注册im系统")
		}
	}

	//2、查找是否有群
	var officialGroup model.ImGroup
	officialGroups, err := db.ImGroupFindByParams(db.Db, map[string]interface{}{
		"key":   "kind",
		"value": 1,
	})
	if err != nil {
		return err
	}
	for _, group := range *officialGroups {
		if officialGroup.GroupId != 0 {
			continue
		}
		//统计当前群有多少人
		userGroups, _ := gim.GroupUserFindByParams(db.DbIm, map[string]interface{}{
			"key":   "group_id",
			"value": group.GroupId,
		})
		if len(*userGroups) < 1000 {
			officialGroup = group
		}
	}

	if officialGroup.GroupId == 0 {
		groupName := "官方【" + utils2.AnyToString(len(*officialGroups)+1) + "】群"
		//3、创建群
		resp, err := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).CreateGroup(utils.GetCtx("", "1", "0"), &pb.CreateGroupReq{
			Name:      groupName,
			AvatarUrl: "",
			//Introduction: "官方群",
			Introduction: "",
			Extra:        "",
			MemberIds:    []int64{gimUser.Id},
		})
		if err != nil {
			return err
		}

		//插入 dou_shen_im_group 记录
		now := time.Now()
		_, err = db.ImGroupInsert(db.Db, &model.ImGroup{
			Kind:       1,
			Uid:        int(1),
			GroupId:    int(resp.GroupId),
			IsFull:     0,
			Name:       groupName,
			CreateTime: now,
			UpdateTime: now,
		})
		if err != nil {
			return err
		}
	} else {
		userGroup, err2 := gim.GroupUserGetOneByParams(db.DbIm, 1, map[string]interface{}{
			"key":   "group_id",
			"value": officialGroup.GroupId,
		})
		if err2 != nil {
			return err2
		}
		if userGroup == nil {
			return errors.New("当前官方群暂未设置群主，请联系管理员！！！")
		}

		//加入群
		_, err11 := utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(
			utils.GetCtx("", strconv.FormatInt(userGroup.UserId, 10), ""),
			&pb.AddGroupMembersReq{
				GroupId: int64(officialGroup.GroupId),
				UserIds: []int64{gimUser.Id},
			})
		if err11 != nil {
			return err11
		}
	}
	return nil
}
